/*
 * Atom.h
 *
 * Copyright Notice: see copyright.txt
 *
 *  Date: 3/8/2012 (D/M/Y)
 *  Author: Dmitrij Lioubartsev
 *  Email: dmitrijl42@gmail.com
 *
 *  This class describes an Atom and its relevant properties,
 *  as well as methods to manipulate and calculate the properties.
 */

#ifndef ATOM_H_
#define ATOM_H_

#define DEFAULT_DIST -1

#include <string>
#include <vector>
using namespace std;

#include "Exclusion.h"

class Atom {
private:
	string element;  //the name of the element, for example "O" or "Fe"
	double x,y,z;	//coordinates
	vector<Atom*> bonds;	//the atom's atomic bonds

	int distToSrc; //used by Substitution algorithms

	double charge;	//the partial charge of this atom
	string ff_type;	//Force Field Type


public:

	Atom(); //default constructor
	Atom(string name, double x, double y, double z);
	~Atom(); //does nothing

	//setters and getters.
	void setx(double x);
	void sety(double y);
	void setz(double z);
	void setElement(string name);
	void setDistToSrc(int dist);
	void setCharge(double charge);
	void setFF_type(string type);

	double getx() const;
	double gety() const;
	double getz() const;
	string getElement();
	int numBonds() {return bonds.size();}
	vector<Atom*>* getBonds() {return &bonds;}
	int getDistToSrc() const;
	double getCharge();
	string getFF_type();

	int operator==(Atom& a) const;


	bool equals(Atom& a);

	void toCubic(double a = 1, double b = 1, double c = 1);

	/*
	 * Adds an atomic bond
	 */
	void addBond(Atom* a) {bonds.push_back(a);}


	bool checkPattern(Exclusion exs);

private:

	//called by checkPattern
	bool hasBond(string el, Atom* notThis = 0);

};


#endif /* ATOM_H_ */
