/*
 * CifParser.h
 *
 * Copyright Notice: see copyright.txt
 *
 * Date: 3/8/2012 (D/M/Y)
 * Author: Dmitrij Lioubartsev
 * Email: dmitrijl42@gmail.com
 *
 * Usage of this class:
 * 1. Call constructor with a pointer to the CifCrystal class
 * 		where the information is to be stored.
 * 2. Call openCif(fileName) to open the .cif file
 * 3. call readcif().
 * 4. ???
 * 5. profit! (all info is now stored in the CifCrystal provided during
 * 				construction, along with various warning messages printed
 * 				if something isn't right)
 */

#ifndef CIFPARSER_H_
#define CIFPARSER_H_

#include "CifCrystal.h"

#include <iostream>
#include <fstream>
using namespace std;

#define CIF_MAX_LINE_LENGTH 2048

class CifParser {

private:
	ifstream cifFile; //file stream
	CifCrystal* crystal; //main crystal structure class
	unsigned int line; //to keep track of the line number.


public:
	CifParser(CifCrystal* structure);
	virtual ~CifParser();

	bool opencif(const char * fileName);
	void readcif();

private: //the following methods are just internal parsing stuff
	bool skipWhiteSpace();
	bool getWord(string& word);
	bool getValue(string& value);
	bool getQuoteValue(string& value, char quote);
	bool isWhiteSpace(char c);
	bool isWhiteSpaceNL(char c);
	bool beginsWith(const char * what, const char * with);
	void setDataItem(string& tag, string& value);
	bool ifReservedReset(string& value);
	void printErrorAS(const char * value);


};

#endif /* CIFPARSER_H_ */
